/*

*/

import tokenizer.*;

/**
   Expression Language E8 but changed to be right associative.

      E ::= T '+' E    // right recursion means right associative
          | T

      T ::= 'a' '*' T  // right recursion means right associative
          | 'a'

   We can rewrite the grammar this way.

      E ::=  T  [ '+' E ]   // right recursion means right associative
      T ::= 'a' [ '*' T ]   // right recursion means right associative

   This parser does not "do" anything.
   It does not create a parse tree nor
   an abstract syntax tree. This parser
   is just a "recognizer" for the language.
   It will throw an error if the string cannot
   be parsed.

   Some example strings are:
      "a + a"
      "a * a"
      "a + a * a"
      "a * a + a"
      "a + a * a + a"
      "a * a + a * a"
      "a * a * a * a"
      "a * a * a + a * a"
*/
public class Parse_E8
{
   public static void recognizeE8(String stringInE8)
   {
      // Tokenize the input string.
      final Tokenizer tokens = new Tokenizer(stringInE8);
      // Parse the list of tokens.
      getE(tokens);
      // There shouldn't be any remaining tokens.
      if ( ! tokens.isEmpty() )
      {
         throw new ParseError("unexpected tokens, found " + tokens.peek());
      }
   }


   /**
      E ::=  T  [ '+' E ]
   */
   private static void getE(Tokenizer tokens)
   {
      getT(tokens);
      if ( ! tokens.isEmpty() && tokens.match("+") )
      {
         getE(tokens); // recursion
      }
   }


   /**
      T ::= 'a' [ '*' T ]
   */
   private static void getT(Tokenizer tokens)
   {
      if ( ! tokens.match("a") )
      {
         throw new ParseError("expected \"a\", found " + tokens.peek());
      }
      if ( ! tokens.isEmpty() && tokens.match("*") )
      {
         getT(tokens); // recursion
      }
   }


   public static void main(String[] args)
   {
      String[] expressions = {
                              "a + a",
                              "a * a",
                              "a + a * a",
                              "a * a + a",
                              "a + a * a + a",
                              "a * a + a * a",
                              "a * a * a * a",
                              "a * a * a + a * a",
                              "a * + a",
                              "+ a * a",
                              "* a * a",
                              "a + + a",
                              "a * * a",
                              "a * a *",
                              "a + a *",
                              "a * a +"
                             };

      for (final String str : expressions)
      {
         try
         {
            System.out.print( "Parsing: \"" + str + "\"" );
            recognizeE8( str );
            System.out.println( " ... Parsed" );
         }
         catch (TokenizeError e)
         {
            System.out.println( " ... Failed (" + e + ")" );
         }
         catch (ParseError e)
         {
            System.out.println( " ... Failed (" + e + ")" );
         }
      }
   }
}
